@@ -13,6 +13,13 @@ Presentation.pm -    .
 
 ################################################################################
 
+sub trunc_string {
+	my ($s, $len) = @_;
+	return length $s <= $len - 3 ? $s : substr ($s, 0, $len - 3) . '...';
+}
+
+################################################################################
+
 =head1 install_drawer
 
   .       Apache.
@@ -215,6 +222,10 @@ sub draw_hr {
 
 ,   
 
+=item max_len
+
+  .   $conf -> {max_len}.   30.
+
 =item href
 
 (  C<undef>)   	
@@ -223,14 +234,23 @@ sub draw_hr {
 
  /  
 
+=item attributes
+
+ HTML-   C<td>.
+
 =back
 
 =head2 
 
 	draw_text_cell ({ 
-		label  => '$1 000 000',
-		href   => '/?type=bank&action=pillage',
-		target => 'invisible',
+		label   => '$1 000 000',
+		href    => '/?type=bank&action=pillage',
+		target  => 'invisible',
+		max_len => 255,
+		attributes => {
+			width => '1%',
+			align => 'right',
+		}
 	});
 
 =cut
@@ -299,22 +319,36 @@ sub draw_one_cell_table {
 
   --     (recordset).
 
+,   -- .
+
+=head2 
+
+=over
+
+=item off
+
+ ,     .
+
 =head2 
 
 	draw_table (
 	
-		['', '', ''],
+		['', {label => '', off => 0}, ''],
 		
 		sub {		
-			draw_text_cell ({label => $i -> {id}) . 
-			draw_text_cell ({label => $i -> {name}) . 		
+			draw_text_cell ({label => $i -> {id}}) . 
+			draw_text_cell ({label => $i -> {name}, off => 0}) . 
 			draw_row_buttons ({}, [{
 				icon => 'delete', 
 				label => '', 
 				href => "/?type=mytype&action=delete&id=$$i{id}", 
 				confirm => " $$i{name}?"
 			}])				
-		}
+		},
+		
+		$data,
+		
+		{off => @$data == 0}
 		
 	);
 
@@ -377,11 +411,18 @@ sub draw_path {
 
  .
 
+=item off
+
+ ,     .
+
 =back
 
 =head2 
 
-	draw_window_title ({label => ' '})
+	draw_window_title ({
+		label => ' ',
+		off   => $no_title,
+	})
 
 =cut
 
@@ -978,6 +1019,21 @@ sub draw_form_field_select {
 
 ################################################################################
 
+=head1 draw_esc_toolbar
+
+    "".     
+C<draw_form>    C<esc>  C<no_ok>.
+
+=cut
+
+################################################################################
+
+sub draw_esc_toolbar {
+	drawer_call ('draw_esc_toolbar', @_);
+}
+
+################################################################################
+
 =head1 draw_ok_esc_toolbar
 
     ""  "".     
